using System;

namespace TBL.CSharp.SaveData.LSD
{
    public abstract partial class Base
    {
        /// <summary>
        /// PNG 头
        /// </summary>
        public static class PngHeader
        {
            /// <summary>
            /// 标识
            /// </summary>
            public static readonly byte[] Signature = { 0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A };
        }

        /// <summary>
        /// <see cref="Lanzainc.SaveData.LSD"/> 元数据块
        /// </summary>
        public static class Lsd
        {
            /// <summary>
            /// 标识
            /// </summary>
            public static readonly byte[] Signature = { (byte)'.', (byte)'L', (byte)'S', (byte)'D' };

            /// <summary>
            /// 默认端序标记
            /// </summary>
            public static class DefaultEndian
            {
                /// <summary>
                /// 大端序
                /// </summary>
                public const byte BigEndian = 0x0A;

                /// <summary>
                /// 小端序
                /// </summary>
                public const byte LittleEndian = 0x0C;

                /// <summary>
                /// 查询是否为本机字节序
                /// </summary>
                public static bool IsNativeEndian(byte endian) => 
                    BitConverter.IsLittleEndian == (endian == LittleEndian);
            }
        }
        
        /// <summary>
        /// <see cref="Lanzainc.SaveData.LSD"/> 普通数据块
        /// </summary>
        public static class LsdData
        {
            /// <summary>
            /// 标识
            /// </summary>
            public static readonly byte[] Signature = { (byte)'.', (byte)'L', (byte)'D', (byte)'T' };
        }
        
        /// <summary>
        /// <see cref="Lanzainc.SaveData.LSD"/> 运行时文件数据块
        /// </summary>
        public static class LsdRuntimeFile
        {
            /// <summary>
            /// 标识
            /// </summary>
            public static readonly byte[] Signature = { (byte)'.', (byte)'L', (byte)'R', (byte)'F' };

            /// <summary>
            /// 目标目录类型
            /// </summary>
            public static class TargetDirectoryType
            {
                /// <summary>
                /// 用户数据目录
                /// </summary>
                public const byte UserData = 0x3A;

                /// <summary>
                /// 临时目录
                /// </summary>
                public const byte Temp = 0x5A;
            }

            /// <summary>
            /// 压缩方式
            /// </summary>
            public static class CompressionMethod
            {
                /// <summary>
                /// 不压缩
                /// </summary>
                public const byte None = 0x1A;

                /// <summary>
                /// Deflate 算法
                /// </summary>
                public const byte Deflate = 0x1F;

                /// <summary>
                /// Lz4 算法
                /// </summary>
                public const byte Lz4 = 0x22;
            }
        }
        
        /// <summary>
        /// <see cref="Lanzainc.SaveData.LSD"/> ROM 数据块
        /// </summary>
        public static class LsdRom
        {
            /// <summary>
            /// 标识
            /// </summary>
            public static readonly byte[] Signature = { (byte)'.', (byte)'L', (byte)'O', (byte)'M' };
        }

        /// <summary>
        /// PNG 尾
        /// </summary>
        public static class PngEnd
        {
            /// <summary>
            /// 标识
            /// </summary>
            public static readonly byte[] Signature = { (byte)'I', (byte)'E', (byte)'N', (byte)'D' };
        }
    }
}